<?php

/**
* Plugin Name: WooCommerce Payment Gateway - OfficeGuy
* Plugin URI: https://www.myofficeguy.com/developers/
* Description: Accept all major credit cards directly on your WooCommerce site in a seamless and secure checkout environment using OfficeGuy credit card clearing and invoicing.
* Version: 1.2.8
* Author: OfficeGuy Ltd.
* Author URI: https://www.myofficeguy.com/
*
* @package WordPress
* @author OfficeGuy Ltd.
* @since 1.0.1
*/

add_action('plugins_loaded', 'woocommerce_officeguy_commerce_init', 0);

function woocommerce_officeguy_commerce_init() {
	if (!class_exists("WC_Payment_Gateway"))
		return;

    /**
    * OfficeGuy Payments Gateway Class
    */
    class WC_OfficeGuy extends WC_Payment_Gateway {
        public static $log = false;

        function __construct() {
			// Register plugin information
            $this->id		    = 'officeguy';
			$this->method_title	= 'OfficeGuy';
            $this->has_fields   = true;
            $this->supports     = array(
              'products'
            );

            // Create plugin fields and settings
            $this->init_form_fields();
            $this->init_settings();

			// Init default settings
			if (empty($this->settings['maxpayments']))
				$this->settings['maxpayments'] = '1';
			if (empty($this->settings['minamountforpayments']))
				$this->settings['minamountforpayments'] = '0';
			if (empty($this->settings['minamountperpayment']))
				$this->settings['minamountperpayment'] = '0';
			if (empty($this->settings['automaticlanguages']))
				$this->settings['automaticlanguages'] = 'yes';
			if (empty($this->settings['mergecustomers']))
				$this->settings['mergecustomers'] = 'no';
			if (empty($this->settings['draftdocument']))
				$this->settings['draftdocument'] = 'no';
			if (empty($this->settings['paypalreceipts']))
				$this->settings['paypalreceipts'] = 'no';
			if (empty($this->settings['fourdigitsyear']))
				$this->settings['fourdigitsyear'] = 'yes';
			if (empty($this->settings['environment']))
				$this->settings['environment'] = 'www';

			// Get setting values
            foreach ($this->settings as $key => $val)
                $this->$key = $val;

			// Load plugin checkout icon
            $this->icon = PLUGIN_DIR . 'images/cards.png';

            // Add hooks
            add_action('woocommerce_receipt_officeguy',                            array($this, 'receipt_page'));
            add_action('woocommerce_update_options_payment_gateways',              array($this, 'process_admin_options'));
            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));

			if ($this->pci != "yes")
                add_action('wp_enqueue_scripts',                                   array($this, 'add_officeguy_scripts'));
            if ($this->logging == 'yes')
                self::$log = wc_get_logger();
			if ($this->settings['paypalreceipts'] == 'yes')
				add_action('woocommerce_api_request', array($this, 'officeguy_paypal_ipn_forwarder'), 10, 1);
            if (!$this->is_valid_for_use())
                $this->enabled = 'no';
        }

        public function is_valid_for_use() {
		    return in_array(get_woocommerce_currency(), array('ILS', 'USD', 'EUR', 'CAD', 'GBP', 'CHF', 'AUD'));
	    }

        /**
        * Initialize Gateway Settings Form Fields.
        */
        function init_form_fields() {
            $this->form_fields = array(
                'general' => array(
					'title'       => __('General Settings', 'officeguy'),
					'type'        => 'title',
					'description' => '',
				),
				'enabled'       => array(
                    'title'       => __('Enable Payments', 'officeguy'),
                    'label'       => __('Enable OfficeGuy Payments', 'officeguy'),
                    'type'        => 'checkbox',
                    'description' => '',
                    'default'     => 'no'
                ),
                'companyid'     => array(
                    'title'       => __('Company ID', 'officeguy'),
                    'type'        => 'text',
                    'description' => __('This is the OfficeGuy CompanyID generated within OfficeGuy.<br />Visit <a target="_blank" href="https://www.myofficeguy.com/developers/keys/">https://www.myofficeguy.com/developers/keys/</a> for additional information.', 'officeguy'),
                    'default'     => ''
                ),
                'privatekey'    => array(
                    'title'       => __('Private Key', 'officeguy'),
                    'type'        => 'text',
                    'description' => __('This is the API private key generated within OfficeGuy.<br />Visit <a target="_blank" href="https://www.myofficeguy.com/developers/keys/">https://www.myofficeguy.com/developers/keys/</a> for additional information.', 'officeguy'),
                    'default'     => ''
                ),
                'publickey'     => array(
                    'title'       => __('Public Key', 'officeguy'),
                    'type'        => 'text',
                    'description' => __('This is the API public key generated within OfficeGuy.<br />Visit <a target="_blank" href="https://www.myofficeguy.com/developers/keys/">https://www.myofficeguy.com/developers/keys/</a> for additional information.', 'officeguy'),
                    'default'     => ''
                ),
                
                'emaildocument' => array(
                    'title'       => __('Email Document', 'officeguy'),
                    'type'        => 'checkbox',
                    'label'       => __('Email created document to the customer.', 'officeguy'),
                    'description' => __('Checking this will cause OfficeGuy to send the created document (Invoice/Receipt) to the customer on successful payment.', 'officeguy'),
                    'default'     => 'yes'
                ),
				'mergecustomers'  => array(
                    'title'       => __('Merge customers', 'officeguy'),
                    'type'        => 'checkbox',
                    'label'       => __('Merge customers on OfficeGuy according to the WooCommerce customer id / email address', 'officeguy'),
                    'default'     => 'no'
                ),
				'paypalreceipts' => array(
                    'title'       => __('PayPal invoice/receipts', 'officeguy'),
                    'type'        => 'checkbox',
                    'label'       => __('Create invoice/receipts on OfficeGuy following PayPal payments.', 'officeguy'),
					'description' => __('Please note this requires the <a target="_blank" href="https://www.myofficeguy.com/applications/paypal/">PayPal application</a> on OfficeGuy.', 'officeguy'),
                    'default'     => 'no'
                ),
				'maxpayments'	  => array(
                    'title'       => __('Maximum Payments', 'officeguy'),
                    'type'        => 'select',
                    'description' => __('Maximum credit card payments to be allowed', 'officeguy'),
                    'options'	  => array(
									'1' => '1',
									'2' => '2',
									'3' => '3',
									'4' => '4',
									'5' => '5',
									'6' => '6',
									'7' => '7',
									'8' => '8',
									'9' => '9',
									'10' => '10',
									'11' => '11',
									'12' => '12',
									'13' => '13',
									'14' => '14',
									'15' => '15',
									'16' => '16',
									'17' => '17',
									'18' => '18',
									'19' => '19',
									'20' => '20',
									'21' => '21',
									'22' => '22',
									'23' => '23',
									'24' => '24',
									'25' => '25',
									'26' => '26',
									'27' => '27',
									'28' => '28',
									'29' => '29',
									'30' => '30',
									'31' => '31',
									'32' => '32',
									'33' => '33',
									'34' => '34',
									'35' => '35',
									'36' => '36'
					),
					'default'     => '1'
                ),
				'minamountforpayments'   => array(
                    'title'       => __('Minimum payments amount', 'officeguy'),
                    'type'        => 'text',
                    'label'       => __('Minimum amounts for payments', 'officeguy'),
                    'description' => __('This sets the minimum order amount for enabling payments. By default, payments are not restricted by the total order amount.', 'officeguy'),
                    'default'     => '0'
                ),
				'minamountperpayment'   => array(
                    'title'       => __('Minimum amount per payment', 'officeguy'),
                    'type'        => 'text',
                    'label'       => __('Minimum amount per payment', 'officeguy'),
                    'description' => __('This sets the minimum amount for each payment when using payments transaction (installments). For instance, when setting a minimum amount per payment of 100NIS, a 300NIS transaction will allow up to 3 installments, and a 600NIS transactions will allow up to 6 installments.', 'officeguy'),
                    'default'     => '0'
                ),
				'automaticlanguages'   => array(
                    'title'       => __('Automatic document language', 'officeguy'),
                    'type'        => 'checkbox',
                    'label'       => __('Automatic document language', 'officeguy'),
                    'description' => __('This allows producing document matching the current logged user locale. If unchecked, documents are produced in Hebrew.', 'officeguy'),
                    'default'     => 'yes'
                ),

				'userinterface' => array(
					'title'       => __('User Interface', 'officeguy'),
					'type'        => 'title',
					'description' => '',
				),
				'title'         => array(
                    'title'       => __('Title', 'officeguy'),
                    'type'        => 'text',
                    'description' => __('This controls the title which the user sees during checkout.', 'officeguy'),
                    'default'     => __('Credit Card (OfficeGuy)', 'officeguy')
                ),
                'description'   => array(
                    'title'       => __('Description', 'officeguy'),
                    'type'        => 'textarea',
                    'description' => __('This controls the description which the user sees during checkout.', 'officeguy'),
                    'default'     => 'Pay with your credit card via OfficeGuy.'
                ),
				'citizenid'     => array(
                    'title'       => __('Citizen ID', 'officeguy'),
                    'type'        => 'select',
                    'label'       => __('Require customer to enter credit card owner citizen id', 'officeguy'),
                    'description' => __('Please note Citizen ID is required by the Israeli credit companies, unless explictly requested for Citizen ID exempt.', 'officeguy'),
                    'default'     => 'yes',
                    'options'	  => array(
									'required' => __('Field is required', 'officeguy'),
									'yes' => __('Field is optional', 'officeguy'),
									'no' => __('Field is hidden', 'officeguy')
                    )
                ),
                'cvv'           => array(
                    'title'       => __('CVV', 'officeguy'),
                    'type'        => 'select',
                    'label'       => __('Require customer to enter credit card CVV code', 'officeguy'),
                    'description' => __('Please note CVV is required by the Israeli credit companies, unless explictly requested for CVV exempt.', 'officeguy'),
                    'default'     => 'required',
                    'options'	  => array(
									'required' => __('Field is required', 'officeguy'),
									'yes' => __('Field is optional', 'officeguy'),
									'no' => __('Field is hidden', 'officeguy')
                    )
                ),
				'fourdigitsyear'  => array(
                    'title'       => __('Full year display', 'officeguy'),
                    'type'        => 'checkbox',
                    'label'       => __('Show year picker as four digits', 'officeguy'),
                    'default'     => 'yes'
                ),
				
				'advanced' => array(
					'title'       => __('Advanced options', 'officeguy'),
					'type'        => 'title',
					'description' => '',
				),
				'draftdocument'       => array(
                    'title'       => __('Draft documents', 'officeguy'),
                    'type'        => 'checkbox',
                    'label'       => __('Issue documents as draft', 'officeguy'),
                    'default'     => 'no'
                ),
				'testing'       => array(
                    'title'       => __('Testing', 'officeguy'),
                    'type'        => 'checkbox',
                    'label'       => __('Testing mode (payments will not be committed, documents will be created as drafts).', 'officeguy'),
                    'description' => __('Make sure to uncheck this before going live!', 'officeguy'),
                    'default'     => 'no'
                ),
                'pci'   => array(
                    'title'       => __('PCI certification', 'officeguy'),
                    'type'        => 'checkbox',
                    'label'       => __('PCI compliant website', 'officeguy'),
                    'description' => __('Check this if your website if PCI compliant, and credit card numbers can be passed through your servers.<br />If you are not sure, keep this unchecked. When unchecked, credit cards will be tokenized using OfficeGuy PaymentsJS library.', 'officeguy'),
                    'default'     => 'no'
                ),
				'merchantnumber'   => array(
                    'title'       => __('Merchant number', 'officeguy'),
                    'type'        => 'text',
                    'label'       => __('Merchant number', 'officeguy'),
                    'description' => __('Do not use this parameter unless explictly requested by OfficeGuy team for multiple merchants usage.', 'officeguy'),
                    'default'     => ''
                ),
                'logging'       => array(
                    'title'       => __('Logging', 'officeguy'),
                    'type'        => 'checkbox',
                    'label'       => __('Enable logging', 'officeguy'),
                    'description' => __('Checking this will log debugging data.', 'officeguy'),
                    'default'     => 'no'
                ),
                'environment'   => array(
                    'title'       => __('Environment', 'officeguy'),
                    'type'        => 'text',
                    'label'       => __('OfficeGuy environment (internal use)', 'officeguy'),
                    'description' => __('Do not use this parameter unless explictly requested by OfficeGuy team for integration testing purposes.', 'officeguy'),
                    'default'     => 'www'
                )
            );
        }

        /**
        * UI - Admin Panel Options
        */
        function admin_options() { ?>
<h3><?php echo __('OfficeGuy Payments', 'officeguy') ?></h3>
<p>
	<?php echo __('The OfficeGuy Payments Gateway is a simple and powerful checkout solution.<br />The plugin adds credit card fields to the checkout page, and processes the credit card transaction on OfficeGuy.<br />In order to avoid PCI certification requirement, credit card details are tokenized on the end user browser using a JavaScript library, and never transmitted through your servers.', 'officeguy') ?>
</p>
<table class="form-table">
  <?php $this->generate_settings_html(); ?>
</table>
<?php
        }

        /**
        * UI - Payment page fields for OfficeGuy Payments.
        */
        function payment_fields() {
			$ordervalue = round(WC()->cart->cart_contents_total + WC()->cart->tax_total + WC()->cart->shipping_total);

            if ($this->pci != "yes") {
                ?> <script>
                    jQuery(function() {
                        jQuery("form.checkout")
							.off("checkout_place_order", og_checkout_place_order)
							.on("checkout_place_order", og_checkout_place_order);
                    });

					function og_checkout_place_order() {
						if (jQuery("input[name=payment_method]:checked").val() != "officeguy")
							return true;
						var Form = jQuery(this);
						if (Form.data("og-ignorewoocommerceevents") == "1")
							return false;
						if (Form.data("og-ignorewoocommerceevents") == "2")
							return true;
                        if (Form.find("input[name=og-ccnum]").length > 0 && Form.find("input[name=og-ccnum]").val().trim().length == 0)
                            return false;
                        if (Form.find("select[name=og-expmonth]").length > 0 && Form.find("select[name=og-expmonth]").val().length == 0)
                            return false;
                        if (Form.find("select[name=og-expyear]").length > 0 && Form.find("select[name=og-expyear]").val().length == 0)
                            return false;
                        if (Form.find("input[name=og-citizenid][required]").length > 0 && Form.find("input[name=og-citizenid][required]").val().trim().length == 0)
                            return false;
                        if (Form.find("input[name=og-cvv][required]").length > 0 && Form.find("input[name=og-cvv][required]").val().trim().length == 0)
                            return false;


						Form.data("og-ignorewoocommerceevents", "1");
						var Settings = {
							FormSelector: "form.checkout",
							CompanyID: <?php echo $this->companyid == '' ? 0 : $this->companyid ?>,
							APIPublicKey: '<?php echo $this->publickey ?>',
							Environment: '<?php echo $this->environment ?>',
							ResponseLanguage: '<?php echo get_locale() ?>',
							Callback: function(TokenValue) {
								Form.removeClass("processing");
								if (TokenValue != null) {
									Form.data("og-ignorewoocommerceevents", "2");
									Form.submit();
								}
								Form.data("og-ignorewoocommerceevents", "0");
								Form.find("input#og-token").val("");
							}
						};
						if (OfficeGuy.Payments.CreateToken(Settings)) {
							Form.data("og-ignorewoocommerceevents", "0");
							return true;
						}
						Form.addClass("processing");
						return false;
					}
                </script> <?php
            }

            // Description of payment method from settings
            if ($this->description) {
                ?><p><?php echo $this->description; ?></p><?php
            }
            ?>
            <fieldset>
                <!-- Show input boxes for new data -->
                <div class="og-errors"></div>

                <div id="officeguy-new-info">
                    <!-- Credit card number -->
                    <p class="form-row form-row-first">
                        <label for="og-ccnum">
                            <?php echo __('Credit Card number', 'officeguy') ?> <span class="required">*</span>
                        </label>
                        <input type="text" class="input-text" id="og-ccnum" name="og-ccnum" data-og="cardnumber" maxlength="20" autocomplete="off" />
                    </p>
                    <?php if ($this->citizenid != 'no') { ?>
                        <p class="form-row form-row-last">
                            <label for="og-citizenid" title="<?php _e('Citizen ID or Passport number for tourists.', 'officeguy') ?>">
                                <?php _e('Israeli Citizen ID', 'officeguy') ?> [?]</span>
                                <?php if ($this->citizenid == 'required') { ?>
                                    <span class="required">*</span>
                                <?php } ?>
                            </label>
                            <input type="text" class="input-text" id="og-citizenid" name="og-citizenid" maxlength="20" style="width:10em" data-og="citizenid" autocomplete="off" <?php echo $this->citizenid == 'required' ? 'required="required" ' : '' ?>/>
                        </p>
                    <?php } ?>

                    <div class="clear"></div>

                    <!-- Credit card expiration -->
                    <p class="form-row form-row-first">
                        <label for="og-expmonth">
                            <?php echo __('Expiration date', 'officeguy') ?> <span class="required">*</span>
                        </label>
                        <select name="og-expmonth" id="og-expmonth" class="woocommerce-select woocommerce-cc-month" data-og="expirationmonth">
                            <option value="">
                                <?php _e('Month', 'officeguy') ?>
                            </option>
                            <?php for ($i = 1; $i <= 12; $i++) {
                                printf('<option value="%u">%s</option>', $i, $i);
                            } ?>
                        </select>
                        <select name="og-expyear" id="og-expyear" class="woocommerce-select woocommerce-cc-year" data-og="expirationyear">
                            <option value="">
                                <?php _e('Year', 'officeguy') ?>
                            </option>
                            <?php for ($i = date('y'); $i <= date('y') + 15; $i++) {
                                printf('<option value="20%u">' . ($this->fourdigitsyear=='yes'?'20':'') . '%u</option>', $i, $i);
                            } ?>
                        </select>
                    </p>

                    <?php
                    // CVV
                    if ($this->cvv != 'no') { ?>
                        <p class="form-row form-row-last">
                            <label for="og-cvv" title="<?php _e('3 or 4 digits usually found on the signature strip.', 'officeguy') ?>">
                                <?php _e('CVV', 'officeguy') ?> [?]
                                <?php if ($this->cvv == 'required') { ?>
                                    <span class="required">*</span>
                                <?php } ?>
                            </label>
                            <input type="text" class="input-text" id="og-cvv" name="og-cvv" maxlength="4" style="width:10em" data-og="cvv" autocomplete="off" <?php echo $this->cvv == 'required' ? 'required="required" ' : '' ?>/>
                        </p>
                    <?php } ?>

					<?php
                    // Allowed payments
					$allowedpayments = $this->get_max_payments($ordervalue);
					
                    if ($allowedpayments > 1) { ?>

						<p class="form-row form-row-first">
                            <label for="og-paymentscount" title="<?php _e('Credit card payments count', 'officeguy') ?>">
                                <?php _e('Payments', 'officeguy') ?> [?]
                            </label>
                            <select class="woocommerce-select" id="og-paymentscount" name="og-paymentscount">
							<?php for ($i = 1; $i <= $allowedpayments; $i++) {
								printf('<option value="%u">%u</option>', $i, $i);
							} ?>
							</select>
                        </p>
                    <?php } ?>
            </fieldset>
            <?php
        }

		function get_max_payments($ordervalue) {
			$max_payments = round($this->maxpayments);
			if ($this->minamountperpayment != '0')
				$max_payments = min($max_payments, round($ordervalue) / round($this->minamountperpayment));
			if ($this->minamountforpayments != '0' && round($ordervalue) < round($this->minamountforpayments))
				$max_payments = 1;
			return $max_payments;
		}

        /**
        * Process the payment and return the result.
        */
        function process_payment($order_id) {
            global $woocommerce;

            $order = new WC_Order($order_id);

            $expmonth = $this->get_post('og-expmonth');
            if ($expmonth < 10)
                $expmonth = '0' . $expmonth;
            $expyear = $this->get_post('og-expyear');

            $items = array();
            foreach ($order->get_items() as $orderitem) {
                array_push($items, array(
                    'Item' => array(
                        'Name' => $orderitem['name'],
                        //'ExternalIdentifier' => $orderitem['product_id'],
                        'SearchMode' => 'Automatic'
                    ),
                    'Quantity' => $orderitem['qty'],
                    'UnitPrice' => round($order->get_line_total($orderitem, true, true) / $orderitem['qty'], 2),
					'Currency' => $order->get_currency()
                ));
            }
            $shipping = $order->get_shipping_total() + $order->get_shipping_tax();
            if ($shipping != 0) {
                array_push($items, array(
                    'Item' => array(
                        'Name' => $order->get_shipping_method(),
                        'SearchMode' => 'Automatic'
                    ),
                    'Quantity' => 1,
                    'UnitPrice' => round($shipping, 2),
					'Currency' => $order->get_currency()
                ));
            }

			$payments_count = $this->get_post('og-paymentscount');
			if ($payments_count == '')
				$payments_count = '1';
			if (round($payments_count) > 1) {
				$ordervalue = round(WC()->cart->cart_contents_total + WC()->cart->tax_total + WC()->cart->shipping_total);
				$allowedpayments = $this->get_max_payments($ordervalue);
				if ($allowedpayments > round($this->maxpayments)) {
					$order->add_order_note(__('Payment failed', 'officeguy') . ' - ' . 'Invalid payments count');
					wc_add_notice(__('Payment failed', 'officeguy') . ' - ' . 'Invalid payments count', $notice_type = 'error');
					return;
				}
			}
			$document_language = '';
			if ($this->automaticlanguages == 'yes') {
				$locale = get_locale();
				if ($locale == 'en_US' || $locale == 'en')
					$document_language = 'English';
				else if ($locale == 'ar_AR' || $locale == 'ar')
					$document_language = 'Arabic';
                else if ($locale == 'es_ES' || $locale == 'es')
					$document_language = 'Spanish';
				else if ($locale == 'he_IL' || $locale == 'he')
					$document_language = 'Hebrew';
			}

			$customer_name = $order->get_billing_first_name() . ' ' . $order->get_billing_last_name();
			if (!empty($order->get_billing_company()) && $order->get_billing_company() != '')
				$customer_name = $order->get_billing_company() . ' - ' . $customer_name;
			$vatrate = "";
			foreach ($order->get_taxes() as $item_id => $item_tax) {
				$tax_data = $item_tax->get_data();
				$item_tax_rate_id = $tax_data['rate_id'];
				$vatrate = str_replace("%", "", WC_Tax::get_rate_percent($item_tax_rate_id));
			}

            $request = array(
                'Customer' => array(
                    'Name'					=> $customer_name,
                    'EmailAddress'			=> $order->get_billing_email(),
                    'City'					=> $order->get_billing_city(),
                    'Address'				=> $order->get_billing_address_1(),
                    'Phone'					=> $order->get_billing_phone(),
					'ExternalIdentifier'	=> ($order->get_customer_id() == '0' ? '' : $order->get_customer_id()),
					'SearchMode'			=> ($this->mergecustomers == 'yes' ? 'Automatic' : 'None'),
					'CompanyNumber'			=> $this->get_post('og-citizenid')
                ),
                'SingleUseToken' => $this->pci != 'yes' ? $this->get_post('og-token') : null,
                'PaymentMethod' => $this->pci == 'yes' ? array(
                    'CreditCard_Number'             => $this->get_post('og-ccnum'),
                    'CreditCard_CVV'                => $this->get_post('og-cvv'),
                    'CreditCard_CitizenID'          => $this->get_post('og-citizenid'),
                    'CreditCard_ExpirationMonth'    => $expmonth,
                    'CreditCard_ExpirationYear'     => $expyear,
                    'Type'                          => 1
                ) : null,
                'Items'					=> $items,
                'VATIncluded'			=> 'true',
				'VATRate'				=> $vatrate,
                'AuthoriseOnly'			=> ($this->testing != 'no' ? 'true' : 'false'),
				'DraftDocument'			=> ($this->draftdocument != 'no' ? 'true' : 'false'),
                'SendDocumentByEmail'	=> ($this->emaildocument == 'yes' ? 'true' : 'false'),
                'DocumentDescription'	=> __('Order number', 'officeguy') . ': ' . $order_id,
				'Payments_Count'		=> $payments_count,
				'DocumentLanguage'		=> $document_language,
				'MerchantNumber'		=> $this->merchantnumber,
                'Credentials'			=> array(
                    'CompanyID' => $this->companyid,
                    'APIKey' => $this->privatekey,
                )
            );

            // Send request and get response from server
            $response = $this->post_and_get_response($request);

            // Check response
            if ($response['Status'] == 0 && $response['Data']['Payment']['ValidPayment'] == true) {
                // Success
				$remark = __('OfficeGuy payment completed. Auth Number: %s. Last digits: %s. Payment ID: %s. Document ID: %s. Customer ID: %s.', 'officeguy');
				$remark = sprintf($remark, $response['Data']['Payment']['AuthNumber'], $response['Data']['Payment']['PaymentMethod']['CreditCard_LastDigits'], $response['Data']['Payment']['ID'], $response['Data']['DocumentID'], $response['Data']['CustomerID']);
                $order->add_order_note($remark);
                $order->payment_complete();

                // Return thank you redirect
                return array(
                    'result'   => 'success',
                    'redirect' => $this->get_return_url($order),
                );
            }
            else if ($response['Status'] != 0)
            {
                // No response or unexpected response
                $order->add_order_note(__('Payment failed', 'officeguy') . ' - ' . $response['UserErrorMessage']);
                wc_add_notice(__('Payment failed', 'officeguy') . ' - ' . $response['UserErrorMessage'], $notice_type = 'error');
            }
            else { // if ($response['Data']['Payment']['ValidPayment'] == false)
                // Decline
                $order->add_order_note(__('Payment failed', 'officeguy') . ' - ' . $response['Data']['Payment']['StatusDescription']);
                wc_add_notice(__('Payment failed', 'officeguy') . ' - ' . $response['Data']['Payment']['StatusDescription'], $notice_type = 'error');
            }
        }

        /**
        * Check payment details for valid format
        */
        function validate_fields() {
            global $woocommerce;

            // Check for saving payment info without having or creating an account
            if ($this->get_post('saveinfo') && !is_user_logged_in() && !$this->get_post('createaccount')) {
                wc_add_notice(__('Sorry, you need to create an account in order for us to save your payment information.', 'officeguy'), $notice_type = 'error');
                return false;
            }

            if ($this->pci != 'yes') {
                $cardToken = $this->get_post('og-token');
                if (empty($cardToken)) {
                    wc_add_notice(__('Card number is invalid.', 'officeguy'), $notice_type = 'error');
                    return false;
                }
            }
            else
            {
                $cardNumber          = $this->get_post('og-ccnum');
                $cardCSC             = $this->get_post('og-cvv');
                $cardExpirationMonth = $this->get_post('og-expmonth');
                $cardExpirationYear  = $this->get_post('og-expyear');

                // Check card number
                if (empty($cardNumber) || !ctype_digit($cardNumber)) {
                    wc_add_notice(__('Card number is invalid.', 'officeguy'), $notice_type = 'error');
                    return false;
                }

                if ($this->cvv == 'yes'){
                    // Check security code
                    if (!ctype_digit($cardCSC)) {
                        wc_add_notice(__('Card security code is invalid (only digits are allowed).', 'officeguy'), $notice_type = 'error');
                        return false;
                    }
                }

                // Check expiration data
                $currentYear = date('Y');

                if (!ctype_digit($cardExpirationMonth)
                    || !ctype_digit($cardExpirationYear)
                    || $cardExpirationMonth > 12
                    || $cardExpirationMonth < 1
                    || $cardExpirationYear < $currentYear
                    || $cardExpirationYear > $currentYear + 20
                ) {
                    wc_add_notice(__('Card expiration date is invalid', 'officeguy'), $notice_type = 'error');
                    return false;
                }
            }

            return true;
        }

        private static function write_to_log($text) {
            if (self::$log == null)
                return;
            self::$log->add('OfficeGuy', $text . "\r\n");
        }

        /**
        * Send the payment data to the gateway server and return the response.
        */
        private function post_and_get_response($request) {
            global $woocommerce;

            // Encode request
            $url = 'https://' . $this->environment . '.myofficeguy.com/api/billing/payments/charge/';
            $requestlog = json_decode(json_encode($request, JSON_PRETTY_PRINT));
            if (isset($requestlog->PaymentMethod)) {
				$requestlog->PaymentMethod->CreditCard_Number = '';
				$requestlog->PaymentMethod->CreditCard_CVV = '';
			}
            self::write_to_log('Request (' . $url . '): ' . json_encode($requestlog, JSON_PRETTY_PRINT));

            // Send request
            $content = wp_remote_post($url, array(
				'body'			=> json_encode($request),
				'timeout'		=> 60,
				'redirection'	=> 5,
				'httpversion'	=> '1.0',
				'blocking'		=> true,
				'headers'		=> array(
					'Content-Type'		=> 'application/json',
					'Content-Language'	=> get_locale()
				),
				'cookies'		=> array(),
				'ssl_verify'	=> false
            ));

            self::write_to_log('Response: ' . json_encode($content));

            // Quit if it didn't work
            if (is_wp_error($content)) {
                wc_add_notice(__('Problem connecting to server at ', 'officeguy') . $url . ' (' . $content->get_error_message() . ')', $notice_type = 'error');
                return null;
            }

            $body = wp_remote_retrieve_body($content);
            $body = json_decode($body, true);

            // Return response array
            return $body;
        }

        function receipt_page($order) {
            echo '<p>' . __('Thank you for your order.', 'officeguy') . '</p>';
        }

        /**
        * Include jQuery and our scripts
        */
        function add_officeguy_scripts() {
            wp_enqueue_script('jquery');
            wp_enqueue_script('officeguypayments', 'https://' . $this->environment . '.myofficeguy.com/scripts/payments.js');
        }

        /**
        * Get post data if set
        */
        private function get_post($name) {
            if (isset($_POST[$name]))
                return $_POST[$name];
            return null;
        }

		function officeguy_paypal_ipn_forwarder($caller) {
			self::write_to_log('IPN Forwarder: ' . http_build_query(stripslashes_deep($_POST)));
			if ($caller != 'wc_gateway_paypal' && $caller != 'wc_gateway_ppec')
				return true;

			$url = 'https://' . $this->environment . '.myofficeguy.com/api/paypal/ipn/receive/?companyid=' . $this->companyid . '&key=' . $this->privatekey;

			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, $url);
			curl_setopt($ch, CURLOPT_POST, 1);
			curl_setopt($ch, CURLOPT_TIMEOUT, 30000);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query(stripslashes_deep($_POST)));
			$data = curl_exec($ch);
			curl_close($ch);
		}
    }

    if (!class_exists('WC_Payment_Gateway'))
        return;

    DEFINE('PLUGIN_DIR', plugins_url(basename(plugin_dir_path(__FILE__)), basename(__FILE__)) . '/');

    /**
    * Add the gateway to woocommerce
    */
    function add_officeguy_commerce_gateway($methods) {
        $methods[] = 'WC_OfficeGuy';
        return $methods;
    }
    function officeguy_load_plugin_textdomain() {
        load_plugin_textdomain('officeguy', false, dirname(plugin_basename( __FILE__ )) . '/languages/');
    }

	/* Make sure PayPal IPN is installed */
	function officeguy_required_plugins_admin_notice() {
		?><div class="notice notice-error"><p><?php echo __('Creating receipts following PayPal payments requires PayPal IPN plugin for WordPress. Please install it.', 'officeguy') ?> <a href="https://wordpress.org/plugins/paypal-ipn/">https://wordpress.org/plugins/paypal-ipn/</a></p></div><?php
	}

    add_action('plugins_loaded', 'officeguy_load_plugin_textdomain');
	add_filter('woocommerce_payment_gateways', 'add_officeguy_commerce_gateway');
}